@@ -254,14 +254,17 @@ SKAT_Check_RCorr<-function(kernel, r.corr){
 SKAT_Check_Method<-function(method,r.corr){
 
 
-	if(method != "liu"  && method != "davies" && method != "liu.mod" && method != "optimal" && method != "optimal.moment" && method != "adjust"  ){
+	if(method != "liu"  && method != "davies" && method != "liu.mod" && method != "optimal" && method != "optimal.moment" 
+	&& method != "optimal.mod" && method != "adjust" && method != "optimal.adj" && method != "optimal.moment.adj"  ){
 		stop("Invalid method!")
 	}
 	
-	if((method == "optimal" || method =="optimal.moment") && length(r.corr) == 1){
+	if((method == "optimal" || method == "optimal.moment" ) && length(r.corr) == 1){
 		r.corr = (0:10)/10
 		#r.corr = c(0, 0.1^2, 0.2^2, 0.3^2, 0.5^2, 0.5, 1)
-	}	
+	} else if( (method == "optimal.mod" || method == "optimal.adj" || method == "optimal.moment.adj" ) && length(r.corr)==1){
+		r.corr = c(0, 0.1^2, 0.2^2, 0.3^2, 0.5^2, 0.5, 1)
+	}
 	if(method =="optimal"){
 		method="davies"
 	} else if (method =="optimal.moment") {
@@ -297,6 +300,7 @@ SKAT_With_NullModel = function(Z, obj.res, kernel = "linear.weighted", method="d
 
 	if(length(r.corr) > 1 && dim(out.z$Z.test)[2] <= 1){
 		r.corr=0
+		method="davies"
 	}
 
 	if(obj.res$out_type == "C"){
@@ -363,6 +367,7 @@ impute.method = "fixed", r.corr=0, is_check_genotype=TRUE, is_dosage = FALSE, mi
 
 	if(length(r.corr) > 1 && dim(out.z$Z.test)[2] <= 1){
 		r.corr=0
+		method="davies"
 	}
 
 	if(length(r.corr) == 1){
